﻿// Copyright (C) 2011 Oliver Sturm <oliver@oliversturm.com>. All rights reserved.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Runtime.Serialization;

namespace CatchingAllTheTime {
  class Program {
    static void Main(string[] args) {
      try {
        var averages = CalculateAveragesFromExternalData("c:\\temp");
        // ...
      }
      catch (Exception ex) {
        Console.WriteLine("Caught exception: " + ex);
      }
    }

    static List<double> CalculateAveragesFromExternalData(string path) {
      // iterate over files in path, calc averages
      try {
        var result = new List<double>( );
        foreach (var file in Directory.EnumerateFiles(path)) {
          result.Add(CalculateAverageFromFile(file));
        }
        return result;
      }
      catch (Exception ex) {
        // Log?
        throw new AverageCalcFailureException("Something went wrong calculating averages for " + path);
      }
    }

    private static double CalculateAverageFromFile(string file) {
      // import file, then calc averages
      try {
        var data = ImportFile(file);
        return CalculateAveragesFromData(data);
      }
      catch (Exception ex) {
        // Log?
        throw new AverageCalcFailureException("Can't calculate averages");
      }
    }

    private static object ImportFile(string file) {
      // load import config, then import
      try {
        var config = LoadConfigFile( );
        return /* .... */ null; // import not implemented for sample
      }
      catch (Exception ex) {
        // Log?
        throw new ImportFailureException("Import failed");
      }
    }

    private static object LoadConfigFile( ) {
      try {
        return File.ReadAllText("import.cfg");
      }
      catch (Exception ex) {
        // Log?
        throw new ConfigLoadException("Couldn't find my config file");
      }
    }

    private static double CalculateAveragesFromData(object data) {
      return /* ... */ 0.0; // not implemented for sample
    }
  }
}
