﻿// Copyright (C) 2011 Oliver Sturm <oliver@oliversturm.com>. All rights reserved.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CatchingExceptions {
  class Program {
    static void Main(string[] args) {
      SimpleCatchException( );
      Console.WriteLine("\n--------------------\n");

      SimpleCatchDivideByZeroException( );
      Console.WriteLine("\n--------------------\n");

      CatchFinally();
      Console.WriteLine("\n--------------------\n");

      BubbleAndCatch( );
      Console.WriteLine("\n--------------------\n");

    }

    private static void SimpleCatchException( ) {
      try {
        Console.WriteLine(GetValue( ));
      }
      catch (Exception ex) {
        Console.WriteLine("Caught an exception: " + ex.Message);

        // Better for logging:
        Console.WriteLine("Caught an exception: " + ex);
      }
    }

    private static void SimpleCatchDivideByZeroException( ) {
      try {
        Console.WriteLine(GetValue( ));
      }
      catch (DivideByZeroException ex) {
        Console.WriteLine("Caught a divide-by-zero exception: " + ex);
      }
    }

    private static int GetValue( ) {
      int a = 10;
      int b = 0;
      int c = a / b;
      return c;
    }

    private static void CatchFinally( ) {
      decimal accountABalance = 1000.0m;
      decimal accountBBalance = 2000.0m;

      bool bookingComplete = false;
      decimal bookingAmount = 500m;
      decimal oldAccountABalance = accountABalance;
      decimal oldAccountBBalance = accountBBalance;

      try {
        BookAmountFromAToB(ref accountABalance, ref accountBBalance, bookingAmount);
        bookingComplete = true;
      }
      catch (Exception ex) {
        Console.WriteLine("Caught exception: " + ex);
      }
      finally {
        if (!bookingComplete) {
          accountABalance = oldAccountABalance;
          accountBBalance = oldAccountBBalance;
        }
      }

      Console.WriteLine("Account A balance: " + accountABalance);
      Console.WriteLine("Account B balance: " + accountBBalance);
    }

    private static void BookAmountFromAToB(ref decimal accountABalance, ref decimal accountBBalance, decimal bookingAmount) {
      accountABalance -= bookingAmount;
      if (bookingAmount == 500m)
        throw new Exception("Whoa, booking 500 has set booking booker on fire!");
      accountBBalance += bookingAmount;
    }

    private static void BubbleAndCatch( ) {
      try {
        BubbleLevel1( );
      }
      catch (Exception ex) {
        Console.WriteLine("Caught exception: " + ex);
      }
    }

    private static void BubbleLevel1( ) {
      BubbleLevel2( );
    }

    private static void BubbleLevel2( ) {
      BubbleLevel3( );
    }

    private static void BubbleLevel3( ) {
      BubbleLevel4( );
    }

    private static void BubbleLevel4( ) {
      BubbleLevel5( );
    }

    private static void BubbleLevel5( ) {
      Console.WriteLine(GetValue( ));
    }

  }
}
